#!/bin/bash
#
function error() {
    echo "$1"
    exit 1
}

if [ -z "$1" ] ; then
    error "usage: $0 SOL_FILE"
fi

TESTS_DIR="testy"
COMPILER_OPTIONS="-Ciort -vw -gl -Cs10000000"
TESTS=`cd $TESTS_DIR ; ls -1 *.in | sort -n -k 3 -t t`
######
SOL="$1"
EXE="/tmp/a.e"
COMPILE_LOG="/tmp/c.log"

function compile() {
    fpc $COMPILER_OPTIONS "$SOL" -o"$EXE" 
    retcode=$?
    if [ $retcode -ne 0 ] ; then
        error "compile error"
    else
        echo "Compile OK"
    fi
}

function test_solution() {
    tname="$1"
    in_file="$TESTS_DIR/$1"
    out_file="${in_file%.in}.out"
    user_out_file="/tmp/a.out"
    user_err_file="/tmp/a.err"
    $EXE < $in_file > $user_out_file 2> $user_err_file
    retcode=$?
    echo -n "$tname .. "
    if [ $retcode -ne 0 ] ; then
	   echo "RUNTIME ERROR: $retcode"
       echo "STDERR":
       cat $user_err_file
       echo "STDOUT":
       cat $user_out_file
    elif ! diff -q -b "$out_file" "$user_out_file" > /dev/null ; then
	   echo "WRONG ANSWER"
	   diff -b "$out_file" "$user_out_file" 
    elif ! diff -q "$out_file" "$user_out_file" > /dev/null ; then
	   echo "ALMOST OK"
	   diff "$out_file" "$user_out_file" 
    else
	   echo "OK"
    fi
}

compile
for t in $TESTS ; do
    test_solution "$t"
done